from feincms.admin.item_editor import FeinCMSInline


class BaseTemplateContentAdmin(FeinCMSInline):
    def __init__(self, *args, **kwargs):
        self.exclude = list(self.exclude or [])
        if len(self.model.template_choices) <= 1:
            self.exclude.append('template')

        super(BaseTemplateContentAdmin,
              self).__init__(*args, **kwargs)



import re
from django.db import models
from django.template import Context, RequestContext
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _

from .admin import BaseTemplateContentAdmin

snake_case_regex = re.compile('((?<=[a-z0-9])[A-Z]|(?!^)[A-Z](?=[a-z]))')


class TemplateContent(models.Model):
    class Meta(object):
        abstract = True

    template_choices = None

    context_object_name = "content"
    template_folder_name = None

    feincms_item_editor_inline = None

    @classmethod
    def initialize_type(class_, **kwargs):
        # Add a field to represent the chosen template.
        # This must be in initialize_type to allow subclasses
        # to customize the template_choices

        class_._ensure_template_choices(**kwargs)

        default = class_.template_choices[0][0]
        class_.add_to_class(
            'template',
            models.CharField(
                _('template'),
                max_length=255,
                choices=class_.template_choices,
                default=default,
            )
        )

        if class_.feincms_item_editor_inline is None:
            class TemplateContentAdmin(BaseTemplateContentAdmin):
                model = class_

            class_.feincms_item_editor_inline = TemplateContentAdmin

    @classmethod
    def _ensure_template_choices(class_, **kwargs):
        if not class_.template_choices:
            name = class_._generate_template_name(**kwargs)
            class_.template_choices = (
                ('content/%s.html' % name, 'Normal'),
            )

    @classmethod
    def _generate_template_name(class_, **kwargs):
        # convert class name from CamelCasse to snake_case
        class_name = class_.__name__
        name = snake_case_regex.sub(r'_\1', class_name)
        name = name.lower()
        # remove trailing _content if present
        name = name.rsplit("_content", 1)[0]
        return name

    def get_template_names(self, **kwargs):
        for template, name in self.template_choices:
            if self.template == template:
                return [template]

        return [self.template_choices[0][0]]

    def get_context_object_name(self, **kwargs):
        return self.context_object_name

    def get_context_data(self, **kwargs):
        return {self.get_context_object_name(**kwargs): self}

    def get_context(self, **kwargs):
        if 'request' in kwargs.keys():
            return RequestContext(
                kwargs['request'],
                dict_=self.get_context_data(**kwargs),
            )
        else:
            return Context(dict_=self.get_context_data(**kwargs))

    def render(self, **kwargs):
        return render_to_string(
            template_name=self.get_template_names(**kwargs),
            context_instance=self.get_context(**kwargs),
        )



from django.utils.translation import ugettext_lazy as _
from feincms.content.richtext.models import (
    RichTextContent as BaseRichTextContent)
from feincms_template_content.models import TemplateContent


class RichTextContent(TemplateContent, BaseRichTextContent):
    class Meta:
        abstract = True
        verbose_name = _('rich text')
        verbose_name_plural = _('rich texts')



from django.conf import settings

settings.configure(
    DATABASES={
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': ':memory:',
        }
    },
    INSTALLED_APPS=(
        'feincms',
        'feincms.module.page',
        'testapp',
    )
)


from django.test import TestCase
from django.template.context import Context
from django.template import RequestContext
from feincms_template_content.models import TemplateContent
from testapp.models import (
    CreatedTestContent,
    CreatedCustomContent,
)
from mock import Mock
from django.contrib import admin
from feincms.module.page.models import Page


admin.autodiscover()

#TODO:
# test that **kwargs get passed around the methods properly


class TemplateContentTestCase(TestCase):
    def test_folder_name(self):
        tests = (
            ('class', 'class'),
            ('Class', 'class'),
            ('CLASS', 'class'),
            ('ClassName', 'class_name'),
            ('className', 'class_name'),
            ('CLASSName', 'class_name'),
            ('ClassNAME', 'class_name'),
            ('classNAME', 'class_name'),
            ('ClassNAMEExtra', 'class_name_extra'),
            ('classNAMEExtra', 'class_name_extra'),
            ('TestContent', 'test'),
            ('test_content', 'test'),
        )
        for name, check in tests:
            test_type = type(name, (TemplateContent,), {
                '__module__': TemplateContent.__module__,
                'Meta': type("Meta", (object,), {'abstract': True}),
            })
            result = test_type._generate_template_name()
            self.assertEqual(result, check)

    def test_template_field(self):
        template_field = None
        for field in CreatedTestContent._meta.fields:
            if field.name == 'template':
                template_field = field

        assert template_field is not None, 'template field not found'
        self.assertEqual(template_field.default,
                         CreatedTestContent.template_choices[0][0])

    def test_default_template_choices(self):
        self.assertEqual(TemplateContent.template_choices, None)

    def test_auto_template_choices(self):
        self.assertEqual(CreatedTestContent.template_choices,
                         (('content/test.html', 'Normal'),))

    def test_manual_template_choices(self):
        self.assertEqual(CreatedCustomContent.template_choices,
                         (('content/custom/1.html', 't1'),
                          ('content/custom/2.html', 't2')))

    def test_get_context_object_name(self):
        c = CreatedTestContent()
        self.assertEqual(c.get_context_object_name(), 'content')
        mock_name = Mock()
        c.context_object_name = mock_name
        self.assertIs(c.get_context_object_name(), mock_name)

    def test_get_context_data(self):
        c = CreatedTestContent()
        self.assertEqual(c.get_context_data(), {'content': c})
        mock_name = Mock()
        c.context_object_name = mock_name
        self.assertEqual(c.get_context_data(), {mock_name: c})

    def test_get_context(self):
        c = CreatedTestContent()
        data = Mock(spec_set=[])
        c.get_context_data = Mock(spec_set=[], return_value=data)
        ctx = c.get_context()
        self.assertEqual(type(ctx), Context)
        self.assertEqual(len(ctx.dicts), 2)
        self.assertEqual(ctx.dicts[1], data)

    def test_get_context_request(self):
        c = CreatedTestContent()
        request = Mock(spec_set=[])
        ctx = c.get_context(request=request)
        self.assertEqual(type(ctx), RequestContext)

    def test_render(self):
        c = CreatedTestContent()
        self.assertEqual(c.render().strip(), 'test')

    def test_admin_form_hidden_template_field(self):
        inline_class = CreatedTestContent.feincms_item_editor_inline
        inline = inline_class(Page, admin.site)
        self.assertIn('template', inline.exclude)

    def test_admin_form_template_field(self):
        inline_class = CreatedCustomContent.feincms_item_editor_inline
        inline = inline_class(Page, admin.site)
        self.assertNotIn('template', inline.exclude)



from setuptools import setup
import feincms_template_content

setup(
    name='FeinCMS-Template-Content',
    version=feincms_template_content.__version__,
    packages=['feincms_template_content'],
    package_data={
        'feincms_template_content': [
            'templates/content/*html',
        ]
    },
    license='MIT License',
    description='FeinCMS content types with (optionally) selectable templates.',
    long_description=open('README.rst').read(),
    author='Richard Ward',
    author_email='richard@richard.ward.name',
    url='https://github.com/RichardOfWard/feincms-template-content',
    install_requires=['django', 'feincms'],
    test_suite='tests',
    tests_require=['mock'],
)



from feincms_template_content.models import TemplateContent
from feincms.module.page.models import Page

Page.register_templates({
    'title': 'Standard template',
    'path': 'feincms/page.html',
    'regions': (
        ('main', 'Main content area'),
    ),
})


class TestContent(TemplateContent):
    class Meta(object):
        abstract = True


class CustomContent(TemplateContent):
    class Meta(object):
        abstract = True

    template_choices = (
        ("content/custom/1.html", "t1"),
        ("content/custom/2.html", "t2"),
    )

CreatedTestContent = Page.create_content_type(TestContent)
CreatedCustomContent = Page.create_content_type(CustomContent)
